{% if report.income_vs_expenditure %}
<table id="table_income_vs_expenditure" class="table">
    <thead>
    <tr class="tabletitle">
	<th align="center">{{ report.income_vs_expenditure.0.1 }}</th>
	<th align="center">{{ report.income_vs_expenditure.0.2 }}</th>
	<th align="center">{{ report.income_vs_expenditure.0.3 }}</th>
    </tr>
    </thead>
    <tbody>    
{% for item in report.income_vs_expenditure %}
    {% if forloop.first %}
    {% else %}   
    {% if forloop.last %}
    {% else %}        	
    <tr{% if item.0 %} class="tabletotal" {% endif %}>
	<td>{{ item.1 }}</td>
	<td align="right">{{ item.2|money|safe }}</td>
	<td align="right">{{ item.3|money|safe }}</td>
    </tr>
      {% endif %}
    {% endif %}
{% endfor %} 
    </tbody>
</table>   
{% endif %}