<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="skrooge_payee" version="5">
  <MenuBar>
    <Menu name="edit"><text>Edit</text></Menu>
  <Menu name="go"><text>Go</text>
	  <Menu name="other"><text>Other</text>
		  <Separator/>
		  <Action name="view_open_similar_payees" group="go_subgroup_3"/>
	  </Menu>	  
  </Menu>
  <Menu name="tool"><text>Tools</text>
	  <Action name="clean_delete_unused_payees" group="tool_group_2"/>
  </Menu>        
  </MenuBar>	
<ToolBar name="mainToolBar"><text>Main Toolbar</text>
</ToolBar>  
<ActionProperties>
    <Action name="clean_delete_unused_payees" />
</ActionProperties>			 
</kpartgui>
